/**
 *  Copyright (C) 2002-2007  The FreeCol Team
 *
 *  This file is part of FreeCol.
 *
 *  FreeCol is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  FreeCol is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FreeCol.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sf.freecol.client.gui.panel;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JSeparator;

import net.miginfocom.swing.MigLayout;
import net.sf.freecol.client.gui.Canvas;
import net.sf.freecol.client.gui.i18n.Messages;
import net.sf.freecol.common.model.GoodsType;
import net.sf.freecol.common.model.IndianNationType;
import net.sf.freecol.common.model.IndianSettlement;
import net.sf.freecol.common.model.Player;
import net.sf.freecol.common.model.Tension;
import net.sf.freecol.common.model.Unit;
import net.sf.freecol.common.model.UnitType;

/**
 * This panel displays the Native Affairs Advisor.
 */
public final class ReportIndianPanel extends ReportPanel {

    /**
     * The constructor that will add the items to this panel.
     * @param parent The parent of this panel.
     */
    public ReportIndianPanel(Canvas parent) {
        super(parent, Messages.message("reportIndianAction.name"));
        Player player = getMyPlayer();
        reportPanel.setLayout(new MigLayout("wrap 5, fillx", "[]20px[center]", ""));
        for (Player opponent : getGame().getPlayers()) {
            if (opponent.isIndian() && !opponent.isDead() && player.hasContacted(opponent)) {
                buildIndianAdvisorPanel(player, opponent);
            }
        }
        scrollPane.getViewport().setOpaque(false);
        reportPanel.setOpaque(false);
        reportPanel.doLayout();
    }

    /**
     * Describe <code>buildIndianAdvisorPanel</code> method here.
     *
     * @param player a <code>Player</code> value
     * @param opponent a <code>Player</code> value
     */
    private void buildIndianAdvisorPanel(Player player, Player opponent) {

        reportPanel.add(new JLabel(Messages.message("report.indian.nameOfTribe")));
        reportPanel.add(new JLabel(Messages.getNationAsString(opponent)), "span 2, wrap");
        reportPanel.add(new JLabel(Messages.message("report.indian.chieftain")));
        reportPanel.add(new JLabel(opponent.getName()), "span 2, wrap");
        reportPanel.add(new JLabel(Messages.message("report.indian.typeOfSettlements")));
        reportPanel.add(new JLabel(((IndianNationType) opponent.getNationType()).getSettlementTypeAsString()),
                        "span 2, wrap");
        reportPanel.add(new JLabel(Messages.message("report.indian.numberOfSettlements")));
        reportPanel.add(new JLabel(String.valueOf(opponent.getSettlements().size())), "span 2, wrap");
        reportPanel.add(new JLabel(Messages.message("report.indian.tension")+":"));
        reportPanel.add(new JLabel(opponent.getTension(player).toString()), "span 2, wrap");

        reportPanel.add(new JSeparator(JSeparator.HORIZONTAL), "span, growx");
        reportPanel.add(new JLabel(Messages.message("Settlement")), "newline 10, center");
        reportPanel.add(new JLabel(Messages.message("mission")));
        reportPanel.add(new JLabel(Messages.message("report.indian.tension")));
        reportPanel.add(new JLabel(Messages.message("report.indian.skillTaught")));
        reportPanel.add(new JLabel(Messages.message("report.indian.tradeInterests")));

        for (IndianSettlement settlement : opponent.getIndianSettlements()) {
            String settlementName = (settlement.getTile().isExplored())
                ? settlement.getName()
                : Messages.message("indianSettlement.nameUnknown");
            String locationName = settlementName + ((settlement.isCapital()) ? "*" : "");
            JButton settlementButton = getLinkButton(locationName, null, settlement.getTile().getId());
            settlementButton.addActionListener(this);
            reportPanel.add(settlementButton, "newline 15");

            Unit missionary = settlement.getMissionary();
            ImageIcon missionChip = null;
            if (missionary != null) {
                boolean expert = missionary.hasAbility("model.ability.expertMissionary");
                missionChip = new ImageIcon(getLibrary().getMissionChip(missionary.getOwner().getColor(), expert));
            }
            reportPanel.add(new JLabel(missionChip));

            Tension tension = settlement.getAlarm(player);
            reportPanel.add(new JLabel((tension == null)
                                  ? Messages.message("indianSettlement.tensionUnknown")
                                  : tension.toString()));

            JLabel skillLabel = new JLabel();
            UnitType skillType = settlement.getLearnableSkill();
            String skill;
            if (skillType != null) {
                skill = Messages.getName(skillType);
                ImageIcon skillImage = getLibrary().getUnitImageIcon(skillType);
                skillLabel.setIcon(getLibrary().getScaledImageIcon(skillImage, 0.66f));
            } else if (settlement.hasBeenVisited(player)) {
                skill = Messages.message("indianSettlement.skillNone");
            } else {
                skill = Messages.message("indianSettlement.skillUnknown");
            }
            skillLabel.setText(skill);
            reportPanel.add(skillLabel);

            GoodsType[] wantedGoods = settlement.getWantedGoods();
            if (wantedGoods[0] == null) {
                reportPanel.add(new JLabel(Messages.message("indianSettlement.wantedGoodsUnknown")));
            } else {
                JLabel goodsLabel = new JLabel(Messages.getName(wantedGoods[0]));
                goodsLabel.setIcon(getLibrary().getScaledImageIcon(getLibrary().getGoodsImageIcon(wantedGoods[0]), 0.66f));
                String split = "split " + String.valueOf(wantedGoods.length);
                reportPanel.add(goodsLabel, split);
                for (int i = 1; i < wantedGoods.length; i++) {
                    if (wantedGoods[i] != null) {
                        goodsLabel = new JLabel(Messages.getName(wantedGoods[i]));
                        goodsLabel.setIcon(getLibrary().getScaledImageIcon(getLibrary().getGoodsImageIcon(wantedGoods[i]), 0.5f));
                        reportPanel.add(goodsLabel);
                    }
                }
            }
        }
        reportPanel.add(new JSeparator(JSeparator.HORIZONTAL), "newline 10, span, growx");
    }
}
